@@ -7,21 +7,23 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
     test = NULL,
     test_file_seen = character(),
     snap_file_seen = character(),
-    file_changed = FALSE,
+    variants_changed = FALSE,
+    fail_on_new = FALSE,
 
     old_snaps = NULL,
     cur_snaps = NULL,
     new_snaps = NULL,
 
-    initialize = function(snap_dir = "_snaps") {
-      self$snap_dir <- snap_dir
+    initialize = function(snap_dir = "_snaps", fail_on_new = FALSE) {
+      self$snap_dir <- normalizePath(snap_dir, mustWork = FALSE)
+      self$fail_on_new <- fail_on_new
     },
 
     start_file = function(path, test = NULL) {
       self$file <- context_name(path)
       self$test_file_seen <- c(self$test_file_seen, self$file)
 
-      self$file_changed <- FALSE
+      self$variants_changed <- character()
 
       self$old_snaps <- FileSnaps$new(self$snap_dir, self$file, type = "old")
       self$cur_snaps <- FileSnaps$new(self$snap_dir, self$file, type = "cur")
@@ -42,7 +44,8 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
                              load = identity,
                              ...,
                              tolerance = testthat_tolerance(),
-                             variant = NULL) {
+                             variant = NULL,
+                             trace_env = NULL) {
       i <- self$new_snaps$append(self$test, variant, save(value))
 
       old_raw <- self$old_snaps$get(self$test, variant, i)
@@ -58,7 +61,11 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
         )
 
         if (length(comp) > 0L) {
-          self$file_changed <- TRUE
+          self$variants_changed <- union(self$variants_changed, variant)
+        } else {
+          # Use the old value for the new snapshot so the snapshot remains
+          # unchanged if the values compare as equal
+          self$new_snaps$set(self$test, variant, i, old_raw)
         }
         comp
       } else {
@@ -66,17 +73,24 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
         check_roundtrip(value, load(value_enc), ..., tolerance = tolerance)
 
         self$cur_snaps$append(self$test, variant, value_enc)
-        testthat_warn(paste0(
+
+        message <- paste0(
           "Adding new snapshot",
-          if (variant != "_default") paste0(" to variant '", variant, "'"),
+          if (variant != "_default") paste0(" for variant '", variant, "'"),
+          if (self$fail_on_new) " in CI",
           ":\n",
           value_enc
-        ))
+        )
+        if (self$fail_on_new) {
+          fail(message, trace_env = trace_env)
+        } else {
+          testthat_warn(message)
+        }
         character()
       }
     },
 
-    take_file_snapshot = function(name, path, file_equal, variant = NULL) {
+    take_file_snapshot = function(name, path, file_equal, variant = NULL, trace_env = NULL) {
       self$announce_file_snapshot(name)
 
       if (is.null(variant)) {
@@ -84,8 +98,16 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
       } else {
         snap_dir <- file.path(self$snap_dir, variant, self$file)
       }
-      snapshot_file_equal(snap_dir, name, path, file_equal)
+      snapshot_file_equal(
+        snap_test_dir = snap_dir,
+        snap_name = name,
+        path = path,
+        file_equal = file_equal,
+        fail_on_new = self$fail_on_new,
+        trace_env = trace_env
+      )
     },
+
     # Also called from announce_snapshot_file()
     announce_file_snapshot = function(name) {
       self$snap_file_seen <- c(self$snap_file_seen, file.path(self$file, name))
@@ -106,16 +128,19 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
       dir.create(self$snap_dir, showWarnings = FALSE)
 
       self$cur_snaps$write()
-      if (self$file_changed) {
-        self$new_snaps$write()
-      } else {
-        self$new_snaps$delete()
+
+      for (variant in self$new_snaps$variants()) {
+        if (variant %in% self$variants_changed) {
+          self$new_snaps$write(variant)
+        } else {
+          self$new_snaps$delete(variant)
+        }
       }
     },
     end_reporter = function() {
       # clean up if we've seen all files
       tests <- context_name(find_test_scripts(".", full.names = FALSE))
-      if (all(tests %in% self$test_file_seen)) {
+      if (!on_ci() && all(tests %in% self$test_file_seen)) {
         snapshot_cleanup(self$snap_dir,
           test_files_seen = self$test_file_seen,
           snap_files_seen = self$snap_file_seen
@@ -125,6 +150,10 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
 
     is_active = function() {
       !is.null(self$file) && !is.null(self$test)
+    },
+
+    snap_files = function() {
+      dir(self$snap_dir, recursive = TRUE)
     }
   )
 )
@@ -162,9 +191,9 @@ get_snapshotter <- function() {
 #'
 #' @export
 #' @keywords internal
-local_snapshotter <- function(snap_dir = NULL, cleanup = FALSE, .env = parent.frame()) {
+local_snapshotter <- function(snap_dir = NULL, cleanup = FALSE, fail_on_new = FALSE, .env = parent.frame()) {
   snap_dir <- snap_dir %||% withr::local_tempdir(.local_envir = .env)
-  reporter <- SnapshotReporter$new(snap_dir = snap_dir)
+  reporter <- SnapshotReporter$new(snap_dir = snap_dir, fail_on_new = fail_on_new)
   if (!identical(cleanup, FALSE)) {
     warn("`cleanup` is deprecated")
   }
