@@ -247,7 +247,7 @@ test_files_reporter <- function(reporter, .env = parent.frame()) {
   reporters <- list(
     find_reporter(reporter),
     lister, # track data
-    local_snapshotter("_snaps", .env = .env) # for snapshots
+    local_snapshotter("_snaps", fail_on_new = FALSE, .env = .env) # for snapshots
   )
   list(
     multi = MultiReporter$new(reporters = compact(reporters)),
