@@ -1,6 +1,6 @@
 
 test_that("error outside of test_that()", {
-  withr::local_envvar(c(TESTTHAT_PARALLEL = "TRUE"))
+  withr::local_envvar(TESTTHAT_PARALLEL = "TRUE")
   err <- tryCatch(
     suppressMessages(testthat::test_local(
       test_path("test-parallel", "outside"),
